@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2022 Antonio Diaz Diaz.
+   Copyright (C) 2013-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include <cctype>
 #include <cerrno>
 #include <cstdio>
-#include <cstdlib>
 #include <stdint.h>	// for lzlib.h
 #include <unistd.h>
 #include <lzlib.h>
@@ -101,8 +100,8 @@ int delete_members( const Cl_options & cl_opts )
   if( !ad.seekable )
     { show_file_error( ad.namep, "Archive is not seekable." ); return 1; }
   if( ad.lzip_index.file_size() < 3 * header_size )
-    { show_file_error( ad.namep, has_lz_ext( ad.name ) ? posix_lz_msg : posix_msg );
-      return 2; }
+    { show_file_error( ad.namep, has_lz_ext( ad.name ) ?
+                       posix_lz_msg : posix_msg ); return 2; }
   // archive is uncompressed seekable, unless compressed corrupt
 
   Archive_reader ar( ad );		// serial reader
@@ -121,7 +120,7 @@ int delete_members( const Cl_options & cl_opts )
     Tar_header header;
     if( ( retval = ar.read( header, header_size ) ) != 0 )
       { show_file_error( ad.namep, ar.e_msg(), ar.e_code() ); break; }
-    if( !verify_ustar_chksum( header ) )		// error or EOA
+    if( !check_ustar_chksum( header ) )		// error or EOA
       {
       if( block_is_zero( header, header_size ) )	// EOA
         {
@@ -129,7 +128,7 @@ int delete_members( const Cl_options & cl_opts )
           { show_file_error( ad.namep, fv_msg1 ); retval = 2; }
         break;
         }
-      // posix format already verified by archive reader
+      // posix format already checked by archive reader
       show_file_error( ad.namep, bad_hdr_msg );
       retval = 2; break;
       }
