@@ -1,5 +1,5 @@
 /* Tarlz - Archiver with multimember lzip compression
-   Copyright (C) 2013-2022 Antonio Diaz Diaz.
+   Copyright (C) 2013-2023 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include <cctype>
 #include <cerrno>
 #include <cstdio>
-#include <cstdlib>
 #include <stdint.h>	// for lzlib.h
 #include <unistd.h>
 #include <lzlib.h>
@@ -65,7 +64,7 @@ int delete_members_lz( const Cl_options & cl_opts,
       Tar_header header;
       if( ( retval = ar.read( header, header_size ) ) != 0 )
         { show_file_error( ad.namep, ar.e_msg(), ar.e_code() ); goto done; }
-      if( !verify_ustar_chksum( header ) )		// error or EOA
+      if( !check_ustar_chksum( header ) )		// error or EOA
         {
         if( block_is_zero( header, header_size ) )	// EOA
           {
@@ -73,7 +72,7 @@ int delete_members_lz( const Cl_options & cl_opts,
             { show_file_error( ad.namep, fv_msg1 ); retval = 2; }
           goto done;
           }
-        // indexed archive reader does not verify posix format
+        // indexed archive reader does not check posix format
         show_file_error( ad.namep, ( ar.data_pos() > header_size ) ?
                          bad_hdr_msg : posix_lz_msg );
         retval = 2;
@@ -112,7 +111,7 @@ int delete_members_lz( const Cl_options & cl_opts,
       if( !check_skip_filename( cl_opts, name_pending, extended.path().c_str() ) )
         {
         print_removed_prefix( extended.removed_prefix );
-        // verify that members match
+        // check that members match
         if( member_begin != ad.lzip_index.dblock( i ).pos() || !ar.at_member_end() )
           { show_file_error( extended.path().c_str(),
                              "Can't delete: not compressed individually." );
